/*
 *
 *  $Id: pvrusb2-video-ivtv.c 730 2005-11-27 02:16:05Z isely $
 *
 *  Copyright (C) 2005 Mike Isely <isely@pobox.com>
 *  Copyright (C) 2004 Aurelien Alleaume <slts@free.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*

   This module connects the pvrusb2 driver to the I2C chip level
   driver which handles device video processing.  This interface is
   used internally by the driver; higher level code should only
   interact through the interface provided by pvrusb2-hdw.h.

   This source file is specifically designed to interface with the
   saa711x support that is available in ivtv, which is entirely
   different than that which is available in v4l.

*/

#include "pvrusb2-video-ivtv.h"

#ifdef PVR2_ENABLE_SAA7115

#include "pvrusb2-i2c.h"
#include "pvrusb2-video.h"
#include "pvrusb2-hdw-internal.h"
#include "pvrusb2-debug.h"
#include "decoder.h"
#include "ivtv.h"
#include <linux/videodev.h>
#include <linux/video_decoder.h>

struct pvr2_decoder_ivtv {
	struct pvr2_decoder base;
	int mode;
};

#define pvr2_decoder_trace(...) pvr2_trace(PVR2_TRACE_DECODER,__VA_ARGS__)


static int pvr2_decoder_ivtv_enable_output(struct pvr2_decoder_ivtv *dcp)
{
	int val = 1;
	int status;
	struct pvr2_hdw *hdw = dcp->base.hdw;
	pvr2_decoder_trace("pvr2_decoder_enable_output");
	status = pvr2_i2c_saa7115_cmd(hdw,DECODER_ENABLE_OUTPUT,&val);
	if (status) return status;
	hdw->subsys_enabled_mask |= PVR2_SUBSYS_DIGITIZER_RUN;
	return 0;
}


static int pvr2_decoder_ivtv_disable_output(struct pvr2_decoder_ivtv *dcp)
{
	int val = 0;
	int status;
	struct pvr2_hdw *hdw = dcp->base.hdw;
	pvr2_decoder_trace("pvr2_decoder_disable_output");
	status = pvr2_i2c_saa7115_cmd(hdw,DECODER_ENABLE_OUTPUT,&val);
	if (status) return status;
	hdw->subsys_enabled_mask &= ~PVR2_SUBSYS_DIGITIZER_RUN;
	return 0;
}


static int pvr2_decoder_ivtv_set_input(struct pvr2_decoder_ivtv *dcp)
{
	int status;
	struct pvr2_hdw *hdw = dcp->base.hdw;
	int v = 0;
	pvr2_decoder_trace("pvr2_decoder_set_input(%d)",
			   hdw->controls[PVR2_CID_INPUT].value);
	switch(hdw->controls[PVR2_CID_INPUT].value){
	case PVR2_CVAL_INPUT_TV:
		v = 4;
		break;
	case PVR2_CVAL_INPUT_COMPOSITE:
		v = 5;
		break;
	case PVR2_CVAL_INPUT_SVIDEO:
		v = 8;
		break;
	case PVR2_CVAL_INPUT_RADIO:
		// ????? No idea yet what to do here
	default:
		return -EINVAL;
	}
	status = pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_INPUT,&v);
	if (!status) {
		hdw->subsys_enabled_mask |= PVR2_SUBSYS_DIGITIZER_CFG_INPUT;
	}
	return status;
}


static int pvr2_decoder_ivtv_set_bcsh(struct pvr2_decoder_ivtv *dcp)
{
	struct decoder_state st;
	struct pvr2_hdw *hdw = dcp->base.hdw;
	int ret;
	pvr2_decoder_trace("pvr2_decoder_set_bcsh b=%d c=%d s=%d h=%d",
			   hdw->controls[PVR2_CID_BRIGHTNESS].value,
			   hdw->controls[PVR2_CID_CONTRAST].value,
			   hdw->controls[PVR2_CID_SATURATION].value,
			   hdw->controls[PVR2_CID_HUE].value);
	st.bright = hdw->controls[PVR2_CID_BRIGHTNESS].value;
	st.contrast = hdw->controls[PVR2_CID_CONTRAST].value;
	st.sat = hdw->controls[PVR2_CID_SATURATION].value;
	st.hue = hdw->controls[PVR2_CID_HUE].value;
	ret = -EINVAL;
	/* isely@pobox.com 15-Oct-2005 - Really ugly kludge to attempt to
	   operate with both older and newer types of saa7115 modules. */
	if ((!dcp->mode) || (dcp->mode == 1)) {
		ret = pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_STATE,&st);
		if (!ret) {
			if (!dcp->mode) {
				pvr2_trace(PVR2_TRACE_INFO,
					   "newer saa7115 sensed");
				dcp->mode = 1;
			}
			hdw->subsys_enabled_mask |=
				PVR2_SUBSYS_DIGITIZER_CFG_BCSH;
		}
		if (!ret || (ret != -EINVAL)) return ret;
	}
	if ((!dcp->mode) || (dcp->mode == 2)) {
		ret = pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_PICTURE,&st);
		if (!ret) {
			if (!dcp->mode) {
				pvr2_trace(PVR2_TRACE_INFO,
					   "old saa7115 sensed");
				dcp->mode = 2;
			}
			hdw->subsys_enabled_mask |=
				PVR2_SUBSYS_DIGITIZER_CFG_BCSH;
		}
		if (!ret || (ret != -EINVAL)) return ret;
	}
	return ret;
}


static int pvr2_decoder_ivtv_is_tuned(struct pvr2_decoder_ivtv *dcp)
{
	int status;
	struct pvr2_hdw *hdw = dcp->base.hdw;
	int ret;
	ret = pvr2_i2c_saa7115_cmd(hdw,DECODER_GET_STATUS,&status);
	if (ret < 0) return -EINVAL;
	if (status & DECODER_STATUS_GOOD) return 1;
	return 0;
}


static int pvr2_decoder_ivtv_set_size(struct pvr2_decoder_ivtv *dcp)
{
	int status;
	struct pvr2_hdw *hdw = dcp->base.hdw;
	struct video_window wind;
	wind.width = hdw->controls[PVR2_CID_HRES].value;
	wind.height = hdw->controls[PVR2_CID_VRES].value;
	pvr2_decoder_trace("pvr2_decoder_set_size(%dx%d)",
			   wind.width,wind.height);
	status = pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_SIZE,&wind);
	if (!status) {
		hdw->subsys_enabled_mask |= PVR2_SUBSYS_DIGITIZER_CFG_SIZE;
	}
	return status;
}


static int pvr2_decoder_ivtv_set_audio(struct pvr2_decoder_ivtv *dcp)
{
	int status;
	struct pvr2_hdw *hdw = dcp->base.hdw;
	int val;
	pvr2_decoder_trace("pvr2_decoder_set_audio %d",
			   hdw->controls[PVR2_CID_SRATE].value);
	switch (hdw->controls[PVR2_CID_SRATE].value) {
	default:
	case PVR2_CVAL_SRATE_48:
		val = DECODER_AUDIO_48_KHZ;
		break;
	case PVR2_CVAL_SRATE_44_1:
		val = DECODER_AUDIO_441_KHZ;
		break;
	}
	status = pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_AUDIO,&val);
	if (!status) {
		hdw->subsys_enabled_mask |= PVR2_SUBSYS_DIGITIZER_CFG_AUDIO;
	}
	return status;
}


static int pvr2_decoder_ivtv_set_norm(struct pvr2_decoder_ivtv *dcp)
{
	int status;
	struct pvr2_hdw *hdw = dcp->base.hdw;
	int val;
	pvr2_decoder_trace("pvr2_decoder_set_norm %d",
			   hdw->controls[PVR2_CID_VIDEOSTANDARD].value);
	switch (hdw->controls[PVR2_CID_VIDEOSTANDARD].value) {
	default:
	case PVR2_CVAL_VIDEOSTANDARD_NTSC_M:
		val = VIDEO_MODE_NTSC;
		break;
	case PVR2_CVAL_VIDEOSTANDARD_SECAM_L:
		val = VIDEO_MODE_SECAM;
		break;
	case PVR2_CVAL_VIDEOSTANDARD_PAL_I:
	case PVR2_CVAL_VIDEOSTANDARD_PAL_DK:
	case PVR2_CVAL_VIDEOSTANDARD_PAL_BG:
		val = VIDEO_MODE_PAL;
		break;
	}
	status = pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_NORM,&val);
	if (!status) {
		hdw->subsys_enabled_mask |= PVR2_SUBSYS_DIGITIZER_CFG_NORM;
	}
	return status;
}


static const struct pvr2_decoder_func_table pvr2_decoder_ivtv_table = {
	(pvr2_decoder_func)0,
	(pvr2_decoder_func)pvr2_decoder_ivtv_set_norm,
	(pvr2_decoder_func)pvr2_decoder_ivtv_set_input,
	(pvr2_decoder_func)pvr2_decoder_ivtv_set_size,
	(pvr2_decoder_func)pvr2_decoder_ivtv_set_audio,
	(pvr2_decoder_func)pvr2_decoder_ivtv_set_bcsh,
	(pvr2_decoder_func)pvr2_decoder_ivtv_is_tuned,
	(pvr2_decoder_func)pvr2_decoder_ivtv_enable_output,
	(pvr2_decoder_func)pvr2_decoder_ivtv_disable_output,
};


struct pvr2_decoder *pvr2_decoder_ivtv_create(struct pvr2_hdw *hdw)
{
	/* Attempt to query the decoder - let's see if it will answer */
	int status;
	int ret;
	struct pvr2_decoder_ivtv *dcp;

	ret = pvr2_i2c_saa7115_cmd(hdw,DECODER_GET_STATUS,&status);
	if (ret < 0) return 0; /* Nope, not there */

	/* Yes, it's there.  So initialize... */
	dcp = kmalloc(sizeof(*dcp),GFP_KERNEL);
	if (!dcp) return 0;
	memset(dcp,0,sizeof(*dcp));
	dcp->base.hdw = hdw;
	dcp->base.func_table = &pvr2_decoder_ivtv_table;
	dcp->mode = 0;
	return &dcp->base;
}


#ifdef notdef
// This causes a null pointer crash inside of saa7115.ko
static int pvr2_decoder_set_vbi(struct pvr2_decoder_ivtv *dcp)
{
	int val = 0;
	pvr2_decoder_trace("pvr2_decoder_set_vbi %d",pvr->config == _vbi);
	if (hdw->config == pvr2_config_vbi) {
		if (hdw->video.standard == PVR_STD_NTSC) {
			val = IVTV_SLICED_CAPTION_525;
		} else {
			val = IVTV_SLICED_CAPTION_625;
		}
	}
	return pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_VBI,(void *)val);
}
#endif

#endif /* PVR2_ENABLE_SAA7115 */

/*
  Stuff for Emacs to see, in order to encourage consistent editing style:
  *** Local Variables: ***
  *** mode: c ***
  *** fill-column: 70 ***
  *** tab-width: 8 ***
  *** c-basic-offset: 8 ***
  *** End: ***
  */
